#
# rhn_register.py - GUI front end code for firstboot screen resolution
#
# Copyright 2003 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import string
import gtk
import gobject
import sys
import os
import functions

import gnome, gnome.ui
from gtk import glade

from rhpl.firstboot_gui_window import FirstbootGuiWindow
sys.path.insert(0, "/usr/share/rhn/up2date_client/")
sys.path.insert(1,"/usr/share/rhn")

import rhnreg
import rhnregGui
import up2dateErrors
import messageWindow


from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("up2date")
gtk.glade.bindtextdomain ("up2date")



class RhnWindow(FirstbootGuiWindow, rhnregGui.ActivatePage):
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 109
    moduleName = _("Activate")
    internalName = "Activate"
    windowTitle =  moduleName
    htmlTag = "rhn"
    shortMessage = _("Connect to Red Hat Network")
    needsparent = 1
    needsnetwork = 1

    def __init__(self):
        FirstbootGuiWindow.__init__(self)
        rhnregGui.ActivatePage.__init__(self)
        self.fb = True
        # if were not useing rhn, skip this module
        if not rhnregGui.useRHN():
            self.skipme = 1

    def passInParent(self,parent):
        self.parent = parent

    def getNext(self):
        pass

    def updatePage(self):
        # set the sensitivity of the "use existing entitments properly" on
        # when we get around to showing the screen
        self.updateSubscriptions()
        self.getHardware()
        self.populateProfile()

    def setupScreen(self):

        self.icon = functions.imageFromPath("/usr/share/system-config-display/pixmaps/system-config-display.png")
        self.mainVBox = gtk.VBox()

        internalVBox = gtk.VBox(gtk.FALSE, 10)
        internalVBox.set_border_width(10)

        vbox  = self.activatePagePrepare()

        internalVBox.pack_start(vbox, gtk.TRUE)
        self.mainVBox.pack_start(internalVBox, gtk.TRUE)

    def launch(self, doDebug=None):
        self.doDebug = doDebug
        self.setupScreen()
        return self.mainVBox, self.icon, self.windowTitle

    def grabAndVerifyFields(self):
        return self.activatePageVerify()

    def writeOutNoPackagesMessage(self):
	filename = "/var/tmp/firstboot/%s_firstboot_finish_message" % self.runPriority
        if not os.access("/var/tmp/firstboot", os.R_OK):
            os.mkdir("/var/tmp/firstboot")
        fd = open(filename, "w")
	# insert proper no packages info here
        txt = _("""You have completed installation and system setup.

Unfortunately, your system was not connected to Red Hat Network. It is
important to connect your system so that you can receive the updates and
patches that keep your system secure and supported.


To connect your system to Red Hat Network go to:
Main Menu, select System Tools, then select Red Hat Network or
From the command line: run /usr/bin/up2date as root in order to register

Need help? To contact a customer service representative near you, visit
http://www.redhat.com/contact/

""")
        fd.write(txt)
        fd.close()

    def writeOutNoChannelsMessage(self):
        filename = "/var/tmp/firstboot/%s_firstboot_finish_message" % ( self.runPriority + 1) 
        if not os.access("/var/tmp/firstboot", os.R_OK):
            os.mkdir("/var/tmp/firstboot")
        fd = open(filename, "w")
	# insert proper no packages info here
        txt = _("""You have completed installation and system set up.
                                                                     
                              
Unfortunately, your subscription was not activated because a valid
subscription number was not supplied.
                                                                     
                              
It is important to activate your subscription so you can access the
services that keep your system secure and supported.
                                                                     
                              
Please contact a customer service representative to help get your
subscription activated.  To find a representative near you, visit
http://www.redhat.com/contact

""")                                                                     
                              
        fd.write(txt)
        fd.close()
        
    def writeOutNoServiceMessage(self):
        filename = "/var/tmp/firstboot/%s_firstboot_finish_message" % ( self.runPriority + 1) 
        if not os.access("/var/tmp/firstboot", os.R_OK):
            os.mkdir("/var/tmp/firstboot")
        fd = open(filename, "w")
	# insert proper no packages info here
	txt = _("""You have completed installation and system set up.

Unfortunately, your subscription was not activated because a valid subscription number was not supplied.

It is important to activate your subscription so you can access the services that keep your system secure and supported.

Please contact a customer service representative to help get your subscription activated.  To find a representative near you, visit http://www.redhat.com/contact/""")

        fd.write(txt)
        fd.close()

    def writeOutSuccessMessage(self):
        filename = "/var/tmp/firstboot/%s_firstboot_finish_message" % ( self.runPriority + 1) 
        if not os.access("/var/tmp/firstboot", os.R_OK):
            os.mkdir("/var/tmp/firstboot")
        fd = open(filename, "w")
	# insert proper no packages info here
        txt = _("""Congratulations!
You have completed setup and your subscription was successfully activated.
                                                                                                                                                                              
You are fully enabled to receive the services included in your Red Hat
subscription.""")
        
        fd.write(txt)
        fd.close()
    

    def apply(self, *args):

        ret =  self.grabAndVerifyFields()
        if self.activationNoPackages:
	    self.writeOutNoPackagesMessage()	
        if ret:
            return None

        ret = self.activatePageApply()
	if ret:
	    return None
        
        if self.noChannels:
            self.writeOutNoChannelsMessage()

        if self.serviceNotEnabled:
            self.writeOutNoServiceMessage()
            
        if not self.serviceNotEnabled  \
           and not self.noChannels \
           and not self.activationNoPackages:
            self.writeOutSuccessMessage()
            
	return True

    def grabFocus(self):
        pass

    def setBusyCursor(self):
        pass
        # gtk 1.2 
#        cursor = gtk.cursor_new (GDK.WATCH)
        # gtk 2.0
##        cursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
##        self.parent.set_cursor(cursor)
##        while gtk.events_pending():
##            gtk.mainiteration(FALSE)

    def setArrowCursor(self):
        pass
        # gtk 1.2
#        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        # gtk 2.0
#        pass
#        cursor = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
#        self.parent.set_cursor(cursor)
#        while gtk.events_pending():
#            gtk.mainiteration(gtk.FALSE)

    def fatalError(self, error, wrap=1):
#        self.setArrowCursor()
        # FIXME
        if wrap:
            text = messageWindow.wrap_text(error)
        else:
            text = error

        dlg = messageWindow.ErrorDialog(text)
        return True
#        gtk.mainquit()
	# huh, yeah, dont do that while running in firstboot
#        sys.exit(1)

childWindow = RhnWindow
