#
# rhn_register.py - GUI front end code for firstboot screen resolution
#
# Copyright 2003 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import string
import gtk
import gobject
import sys
import os
import functions

import gnome, gnome.ui
from gtk import glade

from rhpl.firstboot_gui_window import FirstbootGuiWindow
sys.path.insert(0, "/usr/share/rhn/up2date_client/")
sys.path.insert(1,"/usr/share/rhn")

import rhnreg
import rhnregGui
import up2dateErrors
import messageWindow
import rhn_shared


##
## I18N
## 
#import gettext
#gettext.bindtextdomain ("up2date", "/usr/share/locale")
#gettext.textdomain ("up2date")
#_=gettext.dgettext

from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("up2date")
gtk.glade.bindtextdomain ("up2date", "/usr/share/locale")
#gtk.glade.textdomain("up2date")



class RhnWindow(FirstbootGuiWindow, rhnregGui.LoginPage):
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 106
    moduleName = _("Red Hat Login")
    windowTitle = moduleName
    internalName = "Red Hat Login"
    htmlTag = "rhn"
    shortMessage = _("Register with Red Hat Network")
    needsparent = 1
    needsnetwork = 1

    def __init__(self):
        FirstbootGuiWindow.__init__(self)
        rhnregGui.LoginPage.__init__(self)
        rhn_shared.showOptIn = 0
        if not rhnregGui.useRHN():
            self.skipme = 1

    def getNext(self):
        pass

#    def updatePage(self):
#        self.getCaps()

    def writeOutNoNetworkMessage(self):
        filename = "/var/tmp/firstboot/%s_firstboot_finish_message" % ( self.runPriority + 1) 
        if not os.access("/var/tmp/firstboot", os.R_OK):
            os.mkdir("/var/tmp/firstboot")
        fd = open(filename, "w")
	# insert proper no packages info here
        txt = _("""You have completed installation and system setup.
                                                                                                                                                                              
Unfortunately, your system is not networked, so we were unable to
activate your subscription.
                                                                                                                                                                              
It is important to activate your subscription so you can access the
services that keep your system secure and supported.
                                                                                                                                                                              
Please activate from a networked system now by visiting
http://www.redhat.com/activate/
                                                                                                                                                                              
Need help? To contact a customer service representative near you, visit
http://www.redhat.com/contact/

""")
        fd.write(txt)
        fd.close()

    def passInParent(self, parent):
        self.parent = parent

    def setupScreen(self):

        self.icon = functions.imageFromPath("/usr/share/system-config-display/pixmaps/system-config-display.png")
        self.mainVBox = gtk.VBox()

        internalVBox = gtk.VBox(gtk.FALSE, 10)
        internalVBox.set_border_width(10)

        vbox  = self.loginPagePrepare()

        internalVBox.pack_start(vbox, gtk.TRUE)
        self.mainVBox.pack_start(internalVBox, gtk.TRUE)

        rhn_shared.showOptOut = 0

    def launch(self, doDebug=None):
        self.doDebug = doDebug
        self.setupScreen()
        return self.mainVBox, self.icon, self.windowTitle

    #FIXME: this is a cut&paste from up2date/gui.py, need to refactor
    def grabAndVerifyFields(self):
        return self.loginPageVerify()

    def createSystemRegisterRemindFile(self):
        REMIND_FILE="/etc/sysconfig/rhn/rhn_register_remind"
        if not os.access(REMIND_FILE, os.R_OK):
            # touch the file to tell the applet it needs to remind
            # the user to register
            fd = open(REMIND_FILE, "w+")
            fd.close()

    # run this if this module doesn't have a network
    def noNetwork(self):
        self.writeOutNoNetworkMessage()
        self.createSystemRegisterRemindFile()

    def apply(self, *args):
        if self.doDebug:
            print "applying rhn_login_gui"

        ret =  self.grabAndVerifyFields()
        # this is a bit confusing... gnomeDruid stays on the same page if you
        # return True on the advance... but the firstbootnotebook stays on the
        # the same page if it returns None
        rhn_shared.username = self.loginUname
        rhn_shared.password = self.loginPw
        if ret:
            return None


        if self.useExisting:
            rhn_shared.useExisting = True
            self.parent.setPage("rhn_activate_gui")
            try:
                ret = self.loginPageApply()
            except up2dateErrors.CommunicationError, e:
                msg = _("There was a communication error with the server: %s" % e.errmsg)
                msg = msg + "\n\n" + _("Would you like to try changing the network configuration and trying again?")
                dlg = messageWindow.YesNoDialog(msg)
                ret = dlg.getrc()
                if ret:
                    return None
                else:
                    # skip RHN stuff
                    self.parent.setPage("create_user")
	    if ret:
		return None
            return True
        
        if self.optOut:
            rhn_shared.optOut = True
            self.parent.setPage("rhn_optout_gui")
            return True

        if self.newAccount:
            rhn_shared.newAccount = True
            self.parent.setPage("rhn_newaccount_gui")
            return True

        try:
            ret = self.loginPageApply()
        except up2dateErrors.CommunicationError, e:
            msg = _("There was a communication error with the server: %s" % e.errmsg)
            msg = msg + "\n\n" + _("Would you like to try changing the network configuration and trying again?")
            dlg = messageWindow.YesNoDialog(msg)
            ret = dlg.getrc()
            if ret:
                return None
            else:
                # skip RHN stuff
                self.parent.setPage("create_user")
        return ret

    def loginPageOnOptOut(self):
#        print "rhn_log_gui opt out"
        rhn_shared.showOptOut = 1

    def loginPageOnNewAccount(self):
        pass
#        print "try to get to the new account page"

    def grabFocus(self):
        pass

    def setArrowCursor(self):
        # gtk 1.2
#        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        # gtk 2.0
        cursor = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
        self.mainVBox.window.set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(gtk.FALSE)

    def fatalError(self, error, wrap=1):
#        self.setArrowCursor()
        # FIXME
        if wrap:
            text = messageWindow.wrap_text(error)
        else:
            text = error

        dlg = messageWindow.ErrorDialog(text)
        gtk.mainquit()
#        sys.exit(1)

childWindow = RhnWindow
