#!/usr/bin/python
#
# RHN Registration Client
# Copyright (c) 2000-2002 Red Hat, Inc.
#
# Authors:
#     Adrian Likins <alikins@redhat.com>
#     Preston Brown <pbrown@redhat.com>

import rpm
import os, sys
import socket
import string
import time

sys.path.insert(0, "/usr/share/rhn/")
sys.path.insert(1,"/usr/share/rhn/up2date_client")

import up2dateUtils
import up2dateErrors
import rpcServer
import rpmUtils
import capabilities
import urlparse

try:
    import dmi
except ImportError:
    # If we couldn't import a DMI module, we'll use HAL instead.
    dmi = None
    import dbus

from rhpl.translate import _, N_

try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")

# global variables
#SYSID_DIR = /tmp
SYSID_DIR = "/etc/sysconfig/rhn"
SYSID_FILE = "%s/systemid" % SYSID_DIR
SYSID_BU_FILE = "%s/systemid.save" % SYSID_DIR
REMIND_FILE = "%s/rhn_register_remind" % SYSID_DIR

cachedAvailableChannels = None

import config
cfg = config.initUp2dateConfig()

# Where do we keep the CA certificate for RHNS?
# The servers we're talking to need to have their certs
# signed by one of these CA.
rhns_ca_cert = cfg["sslCACert"] or "/usr/share/rhn/RHNS-CA-CERT"

def version():
    return up2dateUtils.version()

# reload the config in case it's changed on disk
def reloadConfig():
    cfg.load()


def validateEmail(email):
    ret = 1

    if len(email) < 6:
        ret = 0
    if not string.find(email, "@") > 0:
        ret = 0
    if not string.find(email, ".") > 0:
        ret = 0

    return ret

def startRhnsd():
    # successful registration.  Try to start rhnsd if it isn't running.
    if os.access("/usr/sbin/rhnsd", os.R_OK|os.X_OK):
        if os.access("/sbin/chkconfig", os.R_OK|os.X_OK):
            os.system("/sbin/chkconfig rhnsd on > /dev/null");
        else:
            print _("Warning: unable to enable rhnsd with chkconfig")

        rc = os.system("/sbin/service rhnsd status > /dev/null")
        if rc:
            os.system("/sbin/service rhnsd start > /dev/null")
	

# product info structure
productInfoHash = {
    "title" : "",
    "first_name" : "",
    "last_name" : "",
    "company" : "",
    "position" : "",
    "address1" : "",
    "address2" : "",
    "city" : "",
    "state" : "",
    "zip" : "",
    "country" : "",
    "phone" : "",
    "fax" : "",
    "contact_email" : 1,
    "contact_mail" : 0,
    "contact_phone" : 0,
    "contact_fax" : 0, 
    "newsletter" : 0,
    "special_offers" : 0
    }


def getOemInfo():
    configFile = cfg["oemInfoFile"] or "/etc/sysconfig/rhn/oeminfo"

    if not os.access(configFile, os.R_OK):
        return {}

    fd = open(configFile, "r")
    L = fd.readlines()

    info = {}
    for i in L:
        i = string.strip(i)
        if i == "":
            continue
        try:
            (key, value) = string.split(i, ':')
        except ValueError:
            raise up2dateErrors.OemInfoFileError(i)
        
        info[key] = string.strip(value)

    return info
    

        
def registered():
    global SYSID_FILE
    return os.access(SYSID_FILE, os.R_OK)

def removeSystemRegisterRemindFile():
    if os.access(REMIND_FILE, os.R_OK):
        os.unlink(REMIND_FILE)

def writeSystemId(systemId):
    global SYSID_FILE
    global SYSID_DIR
    
    if not os.access(SYSID_DIR, os.W_OK):
        try:
            os.mkdir(SYSID_DIR)
        except:
            return 0

    if not os.access(SYSID_DIR, os.W_OK):
        return 0

    if os.access(SYSID_FILE, os.F_OK):
	# already have systemid file there; let's back it up
	try:
	    os.rename(SYSID_FILE, SYSID_BU_FILE)
	except:
	    return 0

    f = open(SYSID_FILE, "w")
    f.write(systemId)
    f.close()

    # newer registratio  clients will create a file indicating that
    # we need to remind the user to register, this removes it
    removeSystemRegisterRemindFile()

    try:
        os.chmod(SYSID_FILE, 0600)
    except:
        return 0

    
    return 1

def get_virt_info():
    """
    This function returns the UUID and virtualization type of this system, if
    it is a guest.  Otherwise, it returns None.  To figure this out, we'll
    use a number of heuristics (list in order of precedence):

       1.  Check /proc/xen/xsd_port.  If exists, we know the system is a
           host; exit.
       2.  Check /sys/hypervisor/uuid.  If exists and is non-zero, we know
           the system is a para-virt guest; exit.
       3.  Check SMBIOS.  If vendor='Xen' and UUID is non-zero, we know the
           system is a fully-virt guest; exit.
       4.  If non of the above checks worked; we know we have a
           non-xen-enabled system; exit. 
    """

    # First, check whether /proc/xen/xsd_port exists.  If so, we know this is
    # a host system.
    try:
        if os.path.exists("/proc/xen/xsd_port"):
            # Ok, we know this is *at least* a host system.  However, it may
            # also be a fully-virt guest.  Check for that next.  If it is, we'll
            # just report that instead since we only support one level of 
            # virtualization.
            (uuid, virt_type) = get_fully_virt_info()
            return (uuid, virt_type)
    except IOError:
        # Failed.  Move on to next strategy.
        pass

    # This is not a virt host system.  Check if it's a para-virt guest.
    (uuid, virt_type) = get_para_virt_info()
    if uuid is not None:
        return (uuid, virt_type)
        
    # This is not a para-virt guest.  Check if it's a fully-virt guest.
    (uuid, virt_type) = get_fully_virt_info()
    if uuid is not None:
        return (uuid, virt_type)

    # If we got here, we have a system that does not have virtualization
    # enabled.
    return (None, None)

def get_para_virt_info():
    """
    This function checks /sys/hypervisor/uuid to see if the system is a 
    para-virt guest.  It returns a (uuid, virt_type) tuple.
    """
    try:
        uuid_file = open('/sys/hypervisor/uuid', 'r')
        uuid = uuid_file.read()
        uuid_file.close()
        uuid = uuid.lower().replace('-', '')
        virt_type = "para"
        return (uuid, virt_type)
    except IOError:
        # Failed; must not be para-virt.
        pass

    return (None, None)

def get_fully_virt_info():
    """
    This function looks in the SMBIOS area to determine if this is a 
    fully-virt guest.  It returns a (uuid, virt_type) tuple.
    """
    try:
        # For RHEL4 systems and lower, we'll try to use DMI.
        if dmi is not None:
            dmi_registry = dmi.DMI()
            vendor = dmi_registry['system']['vendor']
            if vendor.lower() == "xen":
                uuid = dmi_registry['system']['serial']
                uuid = uuid.lower().replace('-', '')
                virt_type = "fully"
                return (uuid, virt_type)
        else:
            # For RHEL5 systems and higher, we'll use HAL.
            bus = dbus.SystemBus()
            device_obj = \
                bus.get_object('org.freedesktop.Hal',
                               '/org/freedesktop/Hal/devices/computer')
            device = dbus.Interface(device_obj, 'org.freedesktop.Hal.Device')
            vendor = device.GetPropertyString('smbios.bios.vendor')
            if vendor.lower() == "xen":
                uuid = device.GetPropertyString('smbios.system.uuid')
                uuid = uuid.lower().replace('-', '')
                virt_type = "fully"
                return (uuid, virt_type)
    except:
        # Failed.  Must not be fully-virt.
        pass

    return (None, None)

def _is_host_uuid(uuid):
    uuid = eval('0x%s' % uuid)
    return long(uuid) == 0L

#bz 451617 rhn_register TUI looks like it can be used for user registration
#Changing welcome message returned from server to account for 
#user creation no longer being allowed from client tools
def changeWelcomeMessage(msg):
    newMessage = "Register a user account with Red Hat. \n" \
        "\t\tYou can register an account by going to \n" \
        "\t\thttps://rhn.redhat.com/rhn/sales/LoginInfo.do \n"
    markerA = "Step 2: "
    markerB = "Step 3: "
    a = msg.find(markerA)
    if a == -1:
        return msg
    b = msg.find(markerB)
    if b == -1:
        return msg
    splitMsgA = msg[:a+len(markerA)]
    splitMsgB = msg[b:]

    return splitMsgA + newMessage + splitMsgB

def welcomeText():
    s = rpcServer.getServer()

    try:
        welcomeMsg = rpcServer.doCall(s.registration.welcome_message)
        return changeWelcomeMessage(welcomeMsg)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)
    

def privacyText():
    s = rpcServer.getServer()

    try:
        return rpcServer.doCall(s.registration.privacy_statement)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def finishMessage(systemId):

#    ret =  (-1, "blippyFoobar", "this is some text\n\n\nmore\n\ntext\\n\nfoo")
#    return ret

    s = rpcServer.getServer()
    try:
        ret =  rpcServer.doCall(s.registration.finish_message, systemId)
        return ret
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def getCaps():
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.welcome_message)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise f

    caps = capabilities.Capabilities()
    response_headers =  s.get_response_headers()
    caps.populate(response_headers)

    # figure out if were missing any needed caps
    caps.validate()
    
def termsAndConditions():
    s = rpcServer.getServer()

    try:
        return rpcServer.doCall(s.registration.terms_and_conditions)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def reserveUser(username, password):
    s = rpcServer.getServer()
    
    try:
        ret = rpcServer.doCall(s.registration.reserve_user, username, password)
    except rpclib.Fault, f:
        if f.faultCode == -3:
            # account already in use
            raise up2dateErrors.ValidationError(f.faultString)
        if f.faultCode == -2:
            raise up2dateErrors.ValidationError(f.faultString)
        elif f.faultCode == -14:
            # too short password
            raise up2dateErrors.ValidationError(f.faultString)
        elif f.faultCode == -15:
            # bad chars in username
            raise up2dateErrors.ValidationError(f.faultString)
        elif f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        elif f.faultCode == -106:
            raise up2dateErrors.ValidationError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

    return ret

def validateRegNum(regNum):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.validate_reg_num,regNum)
    except rpclib.Fault, f:
        if f.faultCode == -16:
            # invalid
            raise up2dateErrors.ValidationError(f.faultString)
        elif f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)


def registerUser(username, password,
                 email = None, orgid = None, orgpassword = None):
    s = rpcServer.getServer()

    try:
        if not email == None:
            if orgid and orgpassword:
                rpcServer.doCall(s.registration.new_user,
                                 username, password, email, orgid, orgpassword)
            else:
                rpcServer.doCall(s.registration.new_user,
                                 username, password, email)
        else:
                rpcServer.doCall(s.registration.new_user, username, password)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)


def registerSystem(username = None, password = None,
                   profileName = None, packages = None,
                   token = None, other = None):
    s = rpcServer.getServer()

    auth_dict =  { "profile_name" : profileName,
                   "os_release" : up2dateUtils.getVersion(),
		   "release_name" : up2dateUtils.getOSRelease(),
                   "architecture" : up2dateUtils.getArch() };

    # dict of other bits to send 
    if other:
        for (key, item) in other.items():
            auth_dict[key] = item

    if token:
        auth_dict["token"] = token
    else:
        auth_dict["username"] = username
        auth_dict["password"] = password


    auth_dict["uuid"] = cfg["uuid"] or ""
    auth_dict["rhnuuid"] = cfg["rhnuuid"] or ""

    try:
        if packages == None:
            ret = rpcServer.doCall(s.registration.new_system,
                         auth_dict)
        else:
            ret = rpcServer.doCall(s.registration.new_system,
                         auth_dict,
                         packages)
    except rpclib.Fault, f:
        if abs(f.faultCode) == 99:
            raise up2dateErrors.DelayError(f.faultString)
        elif abs(f.faultCode) == 60:
            raise up2dateErrors.AuthenticationTicketError(f.faultString)
        elif abs(f.faultCode) == 105:
            raise up2dateErrors.RhnUuidUniquenessError(f.faultString)
        elif abs(f.faultCode) == 16:
            raise up2dateErrors.InvalidProductRegistrationError(f.faultString)
        elif abs(f.faultCode) == 91:
            raise up2dateErrors.InsuffMgmntEntsError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

    return ret


def registerProduct(systemId, productInfo, oemInfo={}):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.register_product,
                         systemId, productInfo)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def updateContactInfo(username, password, productInfo):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.update_contact_info,
                         username, password, productInfo)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def getAvailableChannels(username, password, useCache = False):
                         
    global cachedAvailableChannels
    
    s = rpcServer.getServer()
    server_arch = up2dateUtils.getArch()
    server_version = up2dateUtils.getVersion()
    server_release = up2dateUtils.getRelease()
    
    if cachedAvailableChannels is None or useCache == False:
        try:
            cachedAvailableChannels = rpcServer.doCall(
                                  s.registration.available_eus_channels,
                                                 username, password,
                                                 server_arch, server_version, 
                                                 server_release)
        except rpclib.Fault, f:
            if f.faultCode == 99:
                raise up2dateErrors.DelayError(f.faultString)
            else:
                raise  
        
    return cachedAvailableChannels

def server_supports_eus():
    return cfg["supportsEUS"]
        
def getRemainingSubscriptions(username, passwd, arch, release):
    s = rpcServer.getServer()

#    print "getRemainingSubscriptions: username: %s passwd: %s arch: %s release: %s" % (username, passwd, arch, release)
    try:
        ret = rpcServer.doCall(s.registration.remaining_subscriptions,
                               username, passwd, arch, release)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise     
    return ret

def sendSerialNumber(systemId, num):
    s = rpcServer.getServer()

    try:
        if cfg["oemId"] != None:
            rpcServer.doCall(s.registration.send_serial, systemId, num,
                                       cfg["oemId"])
        else:
            rpcServer.doCall(s.registration.send_serial, systemId, num)
    except rpclib.Fault, f:
        print f
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)


def sendHardware(systemId, hardwareList):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.add_hw_profile, systemId, hardwareList)
    except rpclib.Fault, f:
        print f
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def sendPackages(systemId, packageList):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.add_packages, systemId, packageList)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def listPackages(systemId):
    s = rpcServer.getServer()

    try:
        print rpcServer.doCall(s.registration.list_packages,systemId)
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)


def updatePackages(systemId):
    s = rpcServer.getServer()

    try:
        rpcServer.doCall(s.registration.add_packages,
                         systemId, rpmUtils.getInstalledPackageList())
    except rpclib.Fault, f:
        if f.faultCode == 99:
            raise up2dateErrors.DelayError(f.faultString)
        else:
            raise up2dateErrors.CommunicationError(f.faultString)

def makeNiceServerUrl(server):
    """Raises up2dateErrors.InvalidProtocolError if the server url has a 
    protocol specified and it's not http or https.
    
    """
    protocol, host, path, parameters, query, fragmentIdentifier = urlparse.urlparse(server)
    if protocol is None or protocol == '':
        server = 'https://' + server
        # We must call it again because if there wasn't a protocol the
        # host will be in path
        protocol, host, path, parameters, query, fragmentIdentifier = urlparse.urlparse(server)
    if protocol not in ['https', 'http']:
        raise up2dateErrors.InvalidProtocolError("You specified an invalid "
                                                 "protocol. Only https and "
                                                 "http are allowed.")
    if path is None or path == '' or path == '/':
        path = '/XMLRPC'
    server = urlparse.urlunparse((protocol, host, path, parameters, query, 
                                  fragmentIdentifier))
    # TODO Raise an exception if url isn't valid
    return server



def getServerType(serverUrl=None):
    """Returns 'hosted' if the url points to a known hosted server. Otherwise
    returns 'satellite'.
    
    If serverUrl is not specified, it is read from the config entry 'serverURL'.
    
    """
    if serverUrl is None:

        # serverURL may be a list in the config file, so by default, grab the
        # first element.
        if type(cfg['serverURL']) == type([]):
            serverUrl = cfg['serverURL'][0]
        else:
            serverUrl = cfg['serverURL']

    serverUrl = makeNiceServerUrl(serverUrl)
    protocol, host, path, parameters, query, fragmentIdentifier = \
            urlparse.urlparse(serverUrl)
            
    hosted_whitelist = cfg['hostedWhitelist']
    
    if host in ['xmlrpc.rhn.redhat.com', 'rhn.redhat.com'] or \
       hosted_whitelist is not None and host in hosted_whitelist:
        return 'hosted'
    else:
        return 'satellite'

