#!/usr/bin/python
#
# TUI for RHN Registration
# Copyright (c) 2000-2002 Red Hat, Inc.
#
# Author:
#       Adrian Likins <alikins@redhat.com>
#       Preston Brown <pbrown@redhat.com>

from socket import gethostname
from os import geteuid, chdir
import sys
sys.path.insert(0, "/usr/share/rhn/")
sys.path.insert(1,"/usr/share/rhn/up2date_client")

from rhpl.translate import _, N_

import snack

import signal

from rhn import rpclib

sys.path.append("/usr/share/rhn/up2date")

import rhnreg, hardware
import up2dateAuth
import up2dateErrors
import up2dateUtils
import rpmUtils
import rpcServer

from rhnreg_constants import *

def FatalErrorWindow(screen, errmsg):
    snack.ButtonChoiceWindow(screen, _("Fatal Error"), "%s" % errmsg,
                             [_("OK")])
    screen.finish()
    sys.exit(1)

def WarningWindow(screen, errmsg):
    snack.ButtonChoiceWindow(screen, _("Warning"), "%s" % errmsg,
                             [_("OK")])
    screen.finish()
    
class StartWindow:
    
    def __init__(self, screen, tui):
        self.name = "StartWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()
        toplevel = snack.GridForm(self.screen, _("Register with Red Hat Network"),
                                  1, 2)

        try:
            tb = snack.Textbox(size[0]-10, size[1]-14, rhnreg.welcomeText(), 1)
        except up2dateErrors.CommunicationError, e:
            FatalErrorWindow(self.screen, e.errmsg)

        toplevel.add(tb, 0, 0, padding = (0, 0, 0, 1))


        self.bb = snack.ButtonBar(self.screen,
                                  [(_("Next"), "next"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 1, growx = 1)

        self.g = toplevel

        rhnreg.getCaps()

    def saveResults(self):
        pass


    def run(self):
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button
    

class PrivacyWindow:

    def __init__(self, screen, tui):
        self.name = "PrivacyWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()
        toplevel = snack.GridForm(screen, _("Step 1: Review the Red Hat Privacy Statement"),
                                  1, 2)

        try:
            tb = snack.Textbox(size[0]-10, size[1]-14, rhnreg.privacyText(), 1)
        except up2dateErrors.CommunicationError, e:
            FatalErrorWindow(screen, e.errmsg)

        toplevel.add(tb, 0, 0, padding = (0, 0, 0, 1))

        self.bb = snack.ButtonBar(screen,
                                  [(_("Next"), "next"),
                                   (_("Back"), "back"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 1, growx = 1)
        self.g = toplevel


    def saveResults(self):
        pass

    def run(self):
        self.screen.refresh()
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button

class TermsAndConditionsWindow:
    def __init__(self, screen, tui):
        self.name = "TermsAndConditionsWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()
        toplevel = snack.GridForm(screen, _("Review the Red Hat Network Terms And Conditions"),
                                  1, 2)

        try:
            text = """ \n\n\n This is a sample Terms and Conditions. \n\n\n"""
            print "#FIXME"
            # text  = rhnreg.termsAndConditions()
            tb = snack.Textbox(size[0]-10, size[1]-14, text, 1)
        except up2dateErrors.CommunicationError, e:
            FatalErrorWindow(screen, e.errmsg)

        toplevel.add(tb, 0, 0, padding = (0, 0, 0, 1))

        self.bb = snack.ButtonBar(screen,
                                  [(_("Next"), "next"),
                                   (_("Back"), "back"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 1, growx = 1)
        self.g = toplevel
        
    def saveResults(self):
        pass

    def run(self):
        self.screen.refresh()
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button
    

class InfoWindow:

    def __init__(self, screen, tui):
        self.name = "InfoWindow"
        self.screen = screen
        self.tui = tui
        self.tui.alreadyRegistered = 0
        self.server = self.tui.serverURL
        size = snack._snack.size()
        
        toplevel = snack.GridForm(screen, _("Setting up software updates"),
                                  1, 9)
        # Satellite
        if self.tui.serverType == 'satellite':
            label = snack.Textbox(size[0]-10, 3,
                                  LOGIN_PROMPT % self.server, 
                                  scroll = 0, wrap = 1)
            toplevel.add(label, 0, 0, anchorLeft = 1)

            grid = snack.Grid(2, 3)

            label = snack.Label(LOGIN) 
            grid.setField(label, 0, 0, padding = (0, 0, 1, 0),
                          anchorRight = 1)

            self.userNameEntry = snack.Entry(20)
            self.userNameEntry.set(tui.userName)
            grid.setField(self.userNameEntry, 1, 0, anchorLeft = 1)

            label = snack.Label(PASSWORD)
            grid.setField(label, 0, 1, padding = (0, 0, 1, 0),
                          anchorRight = 1)

            try:
                self.passwordEntry = snack.Entry(20, password = 1)
            except TypeError:
                self.passwordEntry = snack.Entry(20, hidden = 1)
            self.passwordEntry.set(tui.password)
            grid.setField(self.passwordEntry, 1, 1, anchorLeft = 1)

            toplevel.add(grid, 0, 3)

            label = snack.TextboxReflowed(size[0]-10, LOGIN_TIP)
            toplevel.add(label, 0, 4, anchorLeft=1)

        # Hosted
        else:
            label = snack.TextboxReflowed(size[0]-10,HOSTED_LOGIN_PROMPT)
            toplevel.add(label, 0, 0, anchorLeft = 1)

            grid = snack.Grid(2, 3)

            label = snack.Label(HOSTED_LOGIN)
            grid.setField(label, 0, 0, padding = (0, 0, 1, 0),
                          anchorRight = 1)

            self.userNameEntry = snack.Entry(20)
            self.userNameEntry.set(tui.userName)
            grid.setField(self.userNameEntry, 1, 0, anchorLeft = 1)

            label = snack.Label(PASSWORD)
            grid.setField(label, 0, 1, padding = (0, 0, 1, 0),
                          anchorRight = 1)

            try:
                self.passwordEntry = snack.Entry(20, password = 1)
            except TypeError:
                self.passwordEntry = snack.Entry(20, hidden = 1)
            self.passwordEntry.set(tui.password)
            grid.setField(self.passwordEntry, 1, 1, anchorLeft = 1)

            toplevel.add(grid, 0, 3)

            label = snack.TextboxReflowed(size[0]-10, HOSTED_LOGIN_TIP)
            toplevel.add(label, 0, 4, anchorLeft=1)

        # BUTTON BAR
        self.bb = snack.ButtonBar(screen,
                                    [(NEXT, "next"),
                                    (BACK, "back"),
                                    (CANCEL, "cancel")])
        toplevel.add(self.bb, 0, 8, padding = (0, 1, 0, 0),
                     growx = 1)
        self.g = toplevel


    def validateFields(self):
        if self.userNameEntry.value() == "":
            snack.ButtonChoiceWindow(self.screen, _("Error"),
                                     _("You must enter a user name."),
                                     buttons = [_("OK")])
            self.g.setCurrent(self.userNameEntry)
            return 0
        if self.passwordEntry.value() == "":
            snack.ButtonChoiceWindow(self.screen, _("Error"),
                                     _("You must enter a password."),
                                     buttons = [_("OK")])
            self.g.setCurrent(self.passwordEntry)
            return 0
        try:
            self.tui.alreadyRegistered = rhnreg.reserveUser(self.userNameEntry.value(),
                                                            self.passwordEntry.value())
        except up2dateErrors.ValidationError, e:
            snack.ButtonChoiceWindow(self.screen, _("Error"),
                                     _("The server indicated an error:\n") + e.errmsg,
                                     buttons = [_("OK")])
            self.g.setCurrent(self.userNameEntry)
            return 0
        except up2dateErrors.CommunicationError,e:
            FatalErrorWindow(self.screen,
                             _("There was an error communicating with the registration server:\n") + e.errmsg)
        return 1


    def saveResults(self):
        self.tui.userName = self.userNameEntry.value()
        self.tui.password = self.passwordEntry.value()
        
        
    def run(self):
        self.screen.refresh()
        valid = 0
        while not valid:
            result = self.g.run()
            button = self.bb.buttonPressed(result)

            if result == "F12":
                button = "next"

            if button == "next":
                valid = self.validateFields()
            else:
                break

        self.screen.popWindow()
        return button
    
class OSReleaseWindow:

    def __init__(self, screen, tui):
        self.name = "OSReleaseWindow"
        self.screen = screen
        self.tui = tui
        self.size = snack._snack.size()          

        self.selectChannel = False
 
        toplevel = snack.GridForm(self.screen, 
	                 _("Select Operating System Release"), 1, 10)
        self.g = toplevel

        self.ostext = snack.TextboxReflowed(self.size[0]-10, 
	                    _("Operating System version:"))
        toplevel.add(self.ostext, 0, 1, anchorLeft = 1)
        optiontext1 = _("Limited Updates Only")

        if self.tui.limited_updates_button:
            self.limited_updates_button = snack.SingleRadioButton(optiontext1,
                                                None, isOn = 1)
        else:
            self.limited_updates_button = snack.SingleRadioButton(optiontext1,
                                                None)

        toplevel.add(self.limited_updates_button, 0, 2, padding = (0, 1, 0, 1),
                     anchorLeft = 1)

        self.sublabel = snack.Label(_(" Minor Release: "))
        toplevel.add(self.sublabel, 0, 4, anchorLeft = 1)

        self.channelList = snack.Listbox(self.size[1]-22, 1, 
	                         width = self.size[0]-10)
        toplevel.add(self.channelList, 0, 5)

        self.available_channels = rhnreg.getAvailableChannels(
	                self.tui.userName, self.tui.password, self.tui.other)

        for key, value in self.available_channels['channels'].items():
            if key in self.available_channels['receiving_updates']:
                value = value + "*"
            self.channelList.append(" " + value, key)

        CHANNEL_PAGE_TIP = _("\n Tip: Minor releases with a '*' are currently"
	                     " supported by Red Hat.\n\n")

        self.tip = snack.TextboxReflowed(self.size[0]-10, CHANNEL_PAGE_TIP)
        toplevel.add(self.tip, 0, 6, anchorLeft = 1)

        optiontext2 = _("All available updates")

        if self.tui.all_updates_button:
            self.all_updates_button = snack.SingleRadioButton(optiontext2, 
                                            self.limited_updates_button, isOn=1)
        else:
            self.all_updates_button = snack.SingleRadioButton(optiontext2, 
                                            self.limited_updates_button)
            
        toplevel.add(self.all_updates_button, 0, 7, padding = (0, 0, 0, 1),
                     anchorLeft = 1)

        NEXT      = _("Next")
	BACK      = _("Back")
	CANCEL    = _("Cancel")

        self.bb = snack.ButtonBar(screen,
                          [(NEXT, "next"),
                           (BACK, "back"),
                           (CANCEL, "cancel")])
        toplevel.add(self.bb, 0, 9, growx = 1)

        self.screen.refresh()

        

    def run(self):
        self.screen.refresh()
        valid = "cancel"
        while valid == "cancel":
            result = self.g.run()
            button = self.bb.buttonPressed(result)

            if result == "F12":
                button = "next"

            if button == "next":
                valid = self.validateFields()
            else:
                break

        self.screen.popWindow()
        return button

    def validateFields(self):
        msgbox = "ok"
        later_release = False
        if self.limited_updates_button.selected():
            later_release = self.channelList.current() != \
                                 self.available_channels['default_channel']
         
        title = _("Confirm operating system release selection")
        if later_release:
	    CONFIRM_OS_WARNING = _("Your system will be subscribed to %s.\n"
	                     "base software channel. You will not be\n"
		             "able to move this system to an earlier\n"
			     "(you will be able to move to a newer release.)\n"
			     "Are you sure you Would like to Continue?")
            msgbox = snack.ButtonChoiceWindow(self.screen, title,
                         CONFIRM_OS_WARNING % self.channelList.current()
			 , buttons =[_("ok"), _("cancel")])
            return msgbox

        if self.all_updates_button.selected():
	    CONFIRM_OS_ALL_WARNING = _(
	                       "Your system will be subscribed to the base"
                               " software channel to receives all available"
                               " updates.")
            msgbox = snack.ButtonChoiceWindow(self.screen, title,
                         CONFIRM_OS_ALL_WARNING, 
			 buttons =[_("ok"), _("cancel")])
            return msgbox

        return msgbox

    def saveResults(self):
        # if limited updates save the channel and selction
        # for future use
        if self.limited_updates_button.selected():
            self.tui.other['channel'] = self.channelList.current()
            self.tui.limited_updates_button = self.limited_updates_button.selected()
            self.tui.all_updates_button = 0

        # saving data for all updates button
        if self.all_updates_button.selected():
            self.tui.all_updates_button = self.all_updates_button.selected()
            self.tui.limited_updates_button = 0

class HardwareWindow:

    def __init__(self, screen, tui):
        self.name = "HardwareWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()

        #get the virtualization uuid and set it to other.
        (virt_uuid, virt_type) = rhnreg.get_virt_info()
        if not virt_uuid is None:
            self.tui.other['virt_uuid'] = virt_uuid
            self.tui.other['virt_type'] = virt_type

        # read all hardware in
        tui.hardware = hardware.Hardware()
        
        toplevel = snack.GridForm(screen, _("Step 3: Register a System Profile - Hardware"),
                                  1, 8)

        text = snack.TextboxReflowed(size[0]-10, _("A Profile Name is a descriptive name that you choose to identify this System Profile on the Red Hat Network web pages. Optionally, include a computer serial or identification number."))

        toplevel.add(text, 0, 0, anchorLeft = 1)

        grid = snack.Grid(2, 2)

        label = snack.Label(_("Profile name:"))
        grid.setField(label, 0, 0, padding = (0, 0, 1, 0), anchorRight = 1)

        self.profileEntry = snack.Entry(20)
        grid.setField(self.profileEntry, 1, 0, anchorLeft = 1)

        toplevel.add(grid, 0, 1, anchorLeft = 1)
        
        if tui.includeHardware:
            self.hardwareButton = snack.Checkbox(_("Include the following information about hardware and network:"), isOn = 1)
        else:
            self.hardwareButton = snack.Checkbox(_("Include the following information about hardware and network:"))
            
        toplevel.add(self.hardwareButton, 0, 2, padding = (0, 1, 0, 1),
                     anchorLeft = 1)

        grid = snack.Grid(4, 3)

        self.versionLabel = snack.Label(_("Version:"))
        grid.setField(self.versionLabel, 0, 0, padding = (0, 0, 1, 0), anchorRight = 1)

        self.versionLabel2 = snack.Label(up2dateUtils.getVersion())
        tui.version = up2dateUtils.getVersion()
        grid.setField(self.versionLabel2, 1, 0, anchorLeft = 1)

        self.cpuLabel = snack.Label(_("CPU model:"))
        grid.setField(self.cpuLabel, 2, 0, padding = (1, 0, 1, 0), anchorRight = 1)

        for hw in tui.hardware:            
            if hw['class'] == 'CPU':
                self.cpuLabel2 = snack.Label(hw['model'])
                grid.setField(self.cpuLabel2, 3, 0, anchorLeft = 1)
                
        self.hostnameLabel = snack.Label(_("Hostname:"))
        grid.setField(self.hostnameLabel, 0, 1, padding = (0, 0, 1, 0), anchorRight = 1)

        for hw in tui.hardware:
            if hw['class'] == 'NETINFO':
                self.hostnameLabel2 = snack.Label(hw['hostname'])
                grid.setField(self.hostnameLabel2, 1, 1, anchorLeft = 1)

                if tui.profileName != "":
                    self.profileEntry.set(tui.profileName)
                else:
                    self.profileEntry.set(hw['hostname'])

        self.cpuLabel = snack.Label(_("CPU speed:"))
        grid.setField(self.cpuLabel, 2, 1, padding = (1, 0, 1, 0), anchorRight = 1)

        for hw in tui.hardware:            
            if hw['class'] == 'CPU':
                self.speedLabel2 = snack.Label(_("%d MHz") % hw['speed'])
                grid.setField(self.speedLabel2, 3, 1, anchorLeft = 1)

        self.ipLabel = snack.Label(_("IP Address:"))
        grid.setField(self.ipLabel, 0, 2, padding = (0, 0, 1, 0), anchorRight = 1)


        for hw in tui.hardware:
            if hw['class'] == 'NETINFO':
                self.ipLabel2 = snack.Label(hw['ipaddr'])
                grid.setField(self.ipLabel2, 1, 2, anchorLeft = 1)

        self.ramLabel = snack.Label(_("Memory:"))
        grid.setField(self.ramLabel, 2, 2, padding = (1, 0, 1, 0), anchorRight = 1)

        for hw in tui.hardware:
            if hw['class'] == 'MEMORY':
                self.ramLabel2 = snack.Label(_("%s megabytes") % hw['ram'])
                grid.setField(self.ramLabel2, 3, 2, anchorLeft = 1)

        toplevel.add(grid, 0, 4)

        self.additionalHWLabel = snack.TextboxReflowed(size[0]-10, _("Additional hardware information including PCI devices, disk sizes and mount points will be included in the profile."))

        toplevel.add(self.additionalHWLabel, 0, 5, padding = (0, 1, 0, 0),
                     anchorLeft = 1)
        
        # BUTTON BAR
        self.bb = snack.ButtonBar(screen,
                                  [(_("Next"), "next"),
                                   (_("Back"), "back"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 6, padding = (0, 1, 0, 0),
                     growx = 1)

        self.g = toplevel


    def saveResults(self):
        self.tui.profileName = self.profileEntry.value()
        self.tui.includeHardware = self.hardwareButton.selected()

    def run(self):
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button

class PackagesWindow:

    def __init__(self, screen, tui):
        self.name = "PackagesWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()
        toplevel = snack.GridForm(screen, _("Step 3 : Register a System Profile - Packages"),
                                  1, 7)
        self.g = toplevel


        text = snack.TextboxReflowed(size[0]-10, _("RPM information is important to determine what updated software packages are relevant to this system."))

        toplevel.add(text, 0, 1, anchorLeft = 1)

        self.packagesButton = snack.Checkbox(_("Include RPM packages installed on this system in my System Profile"), 1)
        toplevel.add(self.packagesButton, 0, 2, padding = (0, 1, 0, 1),
                     anchorLeft = 1)

        label = snack.Label(_("You may deselect individual packages by unchecking them below."))
        toplevel.add(label, 0, 3, anchorLeft = 1)

        #self.packageList = snack.Listbox(size[1]-18, 1, width = size[0]-10)
        self.packageList = snack.CheckboxTree(size[1]-18, 1)
        toplevel.add(self.packageList, 0, 4)

        # do we need to read the packages from disk?
        if tui.packageList == []:
            self.pwin = snack.GridForm(screen, _("Building Package List"),
                               1, 1)

            self.scale = snack.Scale(40, 100)
            self.pwin.add(self.scale, 0, 0)
            self.pwin.draw()
            self.screen.refresh()
            getInfo = 0
            if rhnreg.cfg['supportsExtendedPackageProfile']:
                getInfo = 1
            tui.packageList = rpmUtils.getInstalledPackageList(getInfo=getInfo)
            self.screen.popWindow()

        for package in tui.packageList:
            self.packageList.append("%s-%s-%s" % (package[0],
                                                  package[1],
                                                  package[2]),
                                                  item = package[0],
                                                  selected = 1)
            
        # BUTTON BAR
        self.bb = snack.ButtonBar(screen,
                                  [(_("Next"), "next"),
                                   (_("Back"), "back"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 5, padding = (0, 1, 0, 0),
                     growx = 1)



    def setScale(self, amount, total):
        self.scale.set(int(((amount * 1.0)/ total) * 100))
        self.pwin.draw()
        self.screen.refresh()
        

    def saveResults(self):
        self.tui.includePackages = self.packagesButton.selected()
        selection = self.packageList.getSelection()
        for pkg in self.tui.packageList:
            if pkg[0] in selection:
                self.tui.selectedPackages.append(pkg)

        
    def run(self):
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button

class SendWindow:

    def __init__(self, screen, tui):
        self.screen = screen
        self.tui = tui
        self.name = "SendWindow"
        size = snack._snack.size()
        
        toplevel = snack.GridForm(screen, _("Send Profile Information to Red Hat Network"),
                                  1, 2)

        text = snack.TextboxReflowed(size[0]-15, _("We are finished collecting information for the System Profile.\n\nPress \"Next\" to send this System Profile to Red Hat Network.  Click \"Cancel\" and no information will be sent.  You can run the registration program later by typing `up2date --register` at the command line."))
        toplevel.add(text, 0, 0)

        # BUTTON BAR
        self.bb = snack.ButtonBar(screen,
                                  [(_("Next"), "next"),
                                   (_("Back"), "back"),
                                   (_("Cancel"), "cancel")])
        toplevel.add(self.bb, 0, 1, padding = (0, 1, 0, 0),
                     growx = 1)

        self.g = toplevel

    def saveResults(self):
        pass


    def run(self):
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button

class FinishWindow:

    def __init__(self, screen, tui):
        self.name = "FinishWindow"
        self.screen = screen
        self.tui = tui
        size = snack._snack.size()
       
        self.pwin = snack.GridForm(screen, _("Sending Profile to Red Hat Network"),
                                   1, 1)

        self.scale = snack.Scale(40, 100)
        self.pwin.add(self.scale, 0, 0)
        self.pwin.draw()
        self.screen.refresh()

        # send the data
        try:
            rhnreg.registerUser(tui.userName, tui.password, tui.email)
        except up2dateErrors.CommunicationError, e:
            FatalErrorWindow(self.screen, _("Problem registering user name:\n") + e.errmsg)
        except:
            FatalErrorWindow(self.screen, _("Problem registering username."))

        self.setScale(1, 5)

        try:
            systemId = rhnreg.registerSystem(tui.userName, tui.password,
                                             tui.profileName, other = tui.other)
        except up2dateErrors.CommunicationError, e:
            FatalErrorWindow(self.screen, _("Problem registering system:\n") + e.errmsg)
        except up2dateErrors.RhnUuidUniquenessError, e:
            FatalErrorWindow(self.screen, _("Problem registering system:\n") + e.errmsg)
        except up2dateErrors.InsuffMgmntEntsError, e:
	    FatalErrorWindow(self.screen, _("Problem registering system:\n") + e.errmsg)
        except:
            FatalErrorWindow(self.screen, _("Problem registering system."))

        # write the system id out.
        if not rhnreg.writeSystemId(systemId):
            FatalErrorWindow(self.screen, _("Problem writing out system id to disk."))

        self.setScale(2, 5)

        # include the info from the oeminfo file as well
        self.oemInfo = rhnreg.getOemInfo()
        
        # dont send if already registered, do send if they have oemInfo
        if ( not self.tui.alreadyRegistered ) or ( len(self.oemInfo)):
            # send product registration information
            if rhnreg.cfg['supportsUpdateContactInfo']:
                #rhnreg.updateContactInfo(self.username, self.password,  self.tui.productInfo,self.tui.oemInfo)
                try:
                    rhnreg.updateContactInfo(tui.userName, tui.password,  self.tui.productInfo)
                except up2dateErrors.CommunicationError, e:
                    FatalErrorWindow(self.screen, _("Problem registering personal information:\n") + e.errmsg)
                except:
                    print sys.exc_info()
                    print sys.exc_type
                    
                    FatalErrorWindow(self.screen, _("Problem registering personal information"))
                    
            else:
                rhnreg.registerProduct(systemId, self.tui.productInfo,self.tui.oemInfo)
                try:
                    rhnreg.registerProduct(systemId, self.tui.productInfo,self.tui.oemInfo)
                except up2dateErrors.CommunicationError, e:
                    FatalErrorWindow(self.screen, _("Problem registering personal information:\n") + e.errmsg)
                except:
                    FatalErrorWindow(self.screen, _("Problem registering personal information"))

        self.setScale(3, 5)

        # maybe upload hardware profile
        if tui.includeHardware:
            try:
                rhnreg.sendHardware(systemId, tui.hardware)
            except up2dateErrors.CommunicationError, e:
                FatalErrorWindow(self.screen,
                                 _("Problem sending hardware profile:\n") + e.errmsg)
            except:
                FatalErrorWindow(self.screen,
                                 _("Problem sending hardware profile."))

        self.setScale(4, 5)

        # build up package list if necessary
        if tui.includePackages:
            try:
                rhnreg.sendPackages(systemId, tui.selectedPackages)
            except up2dateErrors.CommunicationError, e:
                FatalErrorWindow(self.screen, _("Problem sending package list:\n") + e.errmsg)
            except:
                FatalErrorWindow(self.screen, _("Problem sending package list."))

        success_message = _("You have successfully registered this System Profile on Red Hat Network.\n\nPlease visit http://www.redhat.com/network to login and access your Red Hat Network benefits.\n\nTo upgrade your system with the latest product updates, bug fixes and security enhancements, run up2date at the command line, or choose \"Update Agent\" from the panel.")
       
        no_entitlements_message = _("You have completed installation and system set up. Unfortunately, your subscription was not activated because a valid subscription number was not supplied, or your account did not have matching entitlements remaining. It is important to activate your subscription so you can access the services that keep your system secure and supported. Please contact a customer service representative to help get your subscription activated.  To find a representative near you, visit http://www.redhat.com/contact/")
        # See if we have entitlements
        li = None
        try:
            li = up2dateAuth.updateLoginInfo()
            finish_message = success_message
        except rpclib.Fault, f:
            if f.faultCode == -31:
                finish_message = no_entitlements_message
            else:
                raise f
        
        toplevel = snack.GridForm(screen, _("Registration Finished"), 1, 2)
        text = snack.TextboxReflowed(size[0]-11, finish_message)
        toplevel.add(text, 0, 0)
        # BUTTON BAR
        self.bb = snack.ButtonBar(screen,
                                  [(_("Finish"), "next")])
        toplevel.add(self.bb, 0, 1, padding = (0, 1, 0, 0),
                     growx = 1)

        self.g = toplevel

        rhnreg.startRhnsd()

        self.setScale(5, 5)
        self.screen.popWindow()


    def setScale(self, amount, total):
        self.scale.set(int(((amount * 1.0)/ total) * 100))
        self.pwin.draw()
        self.screen.refresh()


    def saveResults(self):
        pass


    def run(self):
        result = self.g.runOnce()
        button = self.bb.buttonPressed(result)

        if result == "F12":
            return "next"
        return button


class Tui:

    def __init__(self, screen):
        self.screen = screen
        self.size = snack._snack.size()
        self.drawFrame()
        self.alreadyRegistered = 0
        try:
            self.serverType = rhnreg.getServerType()
        except up2dateErrors.InvalidProtocolError:
            FatalErrorWindow(screen, _("You specified an invalid protocol." +
                                     "Only https and http are allowed."))
        # if serverUrl is a list in the config, only reference the first one
        # when we need it
        if type(rhnreg.cfg['serverURL']) == type([]):
            self.serverURL = rhnreg.cfg['serverURL'][0]
        else:
            self.serverURL = rhnreg.cfg['serverURL']

        self.windows = [
            StartWindow,
            PrivacyWindow,
#            TermsAndConditionsWindow,
            InfoWindow,
#            OrgWindow,
#            ProductWindow,
            OSReleaseWindow,
            HardwareWindow,
            PackagesWindow,
            SendWindow,
            FinishWindow
            ]
       
    def __del__(self):
        self.screen.finish()


    def drawFrame(self):
        self.welcomeText = (_("Red Hat Network Registration (c) 2000-2001 Red Hat, Inc."))
        self.screen.drawRootText(0, 0, self.welcomeText)
        self.screen.pushHelpLine(_("  <Tab>/<Alt-Tab> between elements  |  <Space> selects  |  <F12> next screen"))


    def initResults(self):
        self.userName = ""
        self.password = ""
        self.enableEmail = 1
        self.email = ""

        self.oemInfo = {}
        self.productInfo = {
            "reg_num" : "",
            "title" : "",
            "first_name" : "",
            "last_name" : "",
            "company" : "",
            "position" : "",
            "address1" : "",
            "address2" : "",
            "city" : "",
            "state" : "",
            "zip" : "",
            "country" : "",
            "phone" : "",
            "fax" : "",
            "contact_email" : 1,
            "contact_mail" : 0,
            "contact_phone" : 0,
            "contact_fax" : 0,
            "contact_special" : 0,
            "contact_newsletter" : 0
            }

        self.other = {}
        
        self.profileName = ""
        self.includeHardware = 1
        self.version = ""
        self.hostname = ""
        self.ip = ""
        self.cpu = ""
        self.speed = 0
        self.ram = ""
        
        self.limited_updates_button = 1
        self.all_updates_button = 0
        self.includePackages = 0
        self.packageList = []
        self.selectedPackages = []

        
    def run(self):
        self.initResults()
        
        try:
            index = 0
            while index < len(self.windows):
                 
                # check if we support eus before calling the instance
                if self.windows[index].__name__ == 'OSReleaseWindow':
		    if not rhnreg.server_supports_eus():
			index = index + 1
			continue
	        
                win = self.windows[index](self.screen, self)
                
                if win.name == 'OSReleaseWindow':
                    channels = rhnreg.getAvailableChannels(self.userName, 
                               self.password, self.other)
                    # If user is not entitled to eus channels or 
                    # if no eus channels are available skip the
                    # choose channel screen
                    channels = channels['channels']
                    if not rhnreg.server_supports_eus() or len(channels) < 1:
                        index = index + 1
                        continue
            

                result = win.run()
                win.saveResults()

                if result == "back":
                    # hardware window
                    if win == self.windows[4] and self.alreadyRegistered:
                        index = index - 2

                    elif win.name == 'OSReleaseWindow':
                        index = index - 2                        
                    else:
                        index = index - 1
                elif result == "next":
                    # info window
                    if win == self.windows[2] and self.alreadyRegistered:
                        index = index + 2
                    else:
                        index = index + 1
                elif result == "cancel":
                    return

        finally:
            self.screen.finish()

        
def main():
    test = 0    
    signal.signal(signal.SIGINT, signal.SIG_IGN)

    if len(sys.argv) > 1:
        if sys.argv[1] == "-t" or sys.argv[1] == "--test":
            test = 1
            
    screen = snack.SnackScreen()

    if not rpcServer.hasSSL():
        if snack.ButtonChoiceWindow(screen, _("SSL Warning"),
                                    _("""Your system libraries do not support SSL (secure) connections.  Any data that you send or receive from Red Hat Network will be transmitted in the clear. Continue anyway?""")) == "cancel":
            screen.finish()
            sys.exit(1)

    if geteuid() != 0 and not test:
        FatalErrorWindow(screen, _("You must run the RHN registration program as root."))


    if rhnreg.registered() and not test:
        if snack.ButtonChoiceWindow(screen, _("Already Registered"),
                                    _("This system appears to already be registered.  Continue anyway?")) == "cancel":
            
            screen.finish()
            sys.exit(1)
        
    tui = Tui(screen)
    tui.run()

    
if __name__ == "__main__":
    main()
